@@ -1,21 +1,31 @@
 package oEdtk::Tracking;
+my ($_TRACK_SIG, $_TRACK_TRK, @_TRACK_JSON);
 
-my ($_TRACK_SIG, $_TRACK_TRK);
+# xxx TO CHANGE : test if SQLite table already exist before creation
 
 BEGIN { 
-	$SIG{'__WARN__'} = sub { 
-		warn $_[0];
+	$SIG{'__WARN__'} = sub {
+		if ($WARN2LOGGER){
+			&logger (4, $_[0]);
+		} else {
+			warn $_[0];
+		}
+		# when Tracking object is defined, $_TRACK_TRK is defined so that events can be catch and sent to tracker
 		if (defined $_TRACK_TRK && $_TRACK_SIG=~/warn/i) {
 			# http://perldoc.perl.org/functions/warn.html
-			$_TRACK_TRK->track('Warn', 1, $_[0]);
+			my $message=$_[0];
+			chomp ($message);
+			$_TRACK_TRK->track('Warn', 1, $message);
 		} 
 	};
 
 	$SIG{'__DIE__'} = sub { 
-		die $_[0];
 		if (defined $_TRACK_TRK) {
-			$_TRACK_TRK->track('Halt', 1, $_[0]);
-		} 
+			my $message=$_[0];
+			chomp ($message);
+			$_TRACK_TRK->track('Halt', 1, $message);
+		}
+		die $_[0];
 	};
 }
 
@@ -26,19 +36,27 @@ use warnings;
 use oEdtk::Main;
 use oEdtk::Config	qw(config_read);
 use oEdtk::DBAdmin	qw(db_connect create_table_TRACKING);
+use oEdtk::logger	qw(logger $WARN2LOGGER);
 use oEdtk::Dict;
 use Config::IniFiles;
 use Sys::Hostname;
 use DBI;
-
+#use JSON;
 use Exporter;
 
-our $VERSION		= 0.8022;
+our $VERSION		= 1.5063;
 our @ISA			= qw(Exporter);
 our @EXPORT_OK		= qw(stats_iddest stats_week stats_month);
-
+#my  @JSON_TRACE;
 
 sub new {
+	# exemple d'appel
+	# 	$TRK = oEdtk::Tracking->new($ARGV[1] // "input file",  
+	#		user => "utest",
+	#		entity => "Society", 
+	#		keys => ['T_CALLID', 'T_TYPE', 'O_TT_TYP', 'O_TT_ID', 'T_PARAM', 'O_DESIGN', 'T_STATUT' ]
+	#		);
+	
 	my ($class, $source, %params) = @_;
 	$source = $source || ($ARGV[1] || $ARGV[0]);
 	if ($source=~/^\-/){
@@ -51,7 +69,7 @@ sub new {
 
 	my $mode = uc($cfg->{'EDTK_TRACK_MODE'});
 	if ($mode eq 'NONE') {
-		warn "INFO : Tracking is currently disabled...\n";
+		&logger (3, "Tracking is currently disabled... (see EDTK_TRACK_MODE in config file)");
 		# Return a dummy object if tracking is disabled.
 		return bless { dict => $dict, mode => $mode }, $class;
 	}
@@ -75,7 +93,7 @@ sub new {
 		my $key = uc($_);
 		if (length($key) > 8) {
 			$key =~ s/^(.{8}).*$/$1/;
-			warn "INFO : column \"\U$_\E\" too long, truncated to \"$key\"\n";
+			&logger (5, "column too long, truncated to '$key'");
 		}
 		if (exists($seen{$key})) {
 			die "ERROR: duplicate column \"$key\"";
@@ -86,11 +104,12 @@ sub new {
 
 	# Extract application name from the script name.
 	my $app = $0;
-	$app =~ s/^.*?[\/\\]?([A-Z0-9-_]+)\.pl$/$1/;
+	 $app =~ s/^.*?[\/\\]?([\w-]+[\.plmex]*$)/$1/;
+	#$app =~ s/^.*?[\/\\]?([A-Z0-9-_]+)\.pl$/$1/;
 	if (length($app) > 20) {
 		$app =~ s/\.pl$//i;
 		$app =~ /(.{20})$/;
-		warn "INFO : application name \"$app\" too long, truncated to \"$1\"\n";
+		&logger (4, "application name '$app' too long, truncated to '$1'");
 		$app = $1;
 	}
 
@@ -101,7 +120,7 @@ sub new {
 	my $user = $params{'user'} || 'None';
 	if (length($user) > 10) {
 		$user =~ s/^(.{10}).*$/$1/;
-		warn "INFO : username \"$params{'user'}\" too long, truncated to \"$user\"\n";
+		&logger (4, "username \"$params{'user'}\" too long, truncated to \"$user\"");
 	}
 
 	# Truncate if necessary, by taking at most 128 characters on the right.
@@ -110,41 +129,62 @@ sub new {
 	}
 
 	my $self = bless {
-		dict	=> $dict,
-		mode	=> $mode,
-		table=> $table,
-		edmode=>$edmode,
-		id	=> oe_ID_LDOC(),
-		seq	=> 1,
-		keys	=> \@userkeys,
-		user	=> $user,
-		source=>$source,
-		app	=> $app,
-		dbh	=> $dbh
+		dict		=> $dict,
+		mode		=> $mode,
+		table	=> $table,
+		edmode	=> $edmode,
+		id		=> oe_ID_LDOC(),
+		seq		=> 1,
+		keys		=> \@userkeys,
+		user		=> $user,
+		source	=> $source,
+		app		=> $app,
+
+		entity 	=> $params{'entity'} // "",
+		account 	=> $params{'account'} // "",
+		obj_typ 	=> $params{'obj_typ'} // "",
+		child_typ => $params{'child_typ'} // "",
+		child_id	=> $params{'child_id'} // "",
+		parent_id => $params{'parent_id'} // "",
+
+		dbh		=> $dbh
 	}, $class;
 
 	my $entity = $params{'entity'} || $cfg->{'EDTK_CORP'};
 	$self->set_entity($entity);
 
-	# Create the table in the SQLite case.
+	# Create the table in the SQLite case
 	if ($dbh->{'Driver'}->{'Name'} eq 'SQLite') {
-		eval { create_table_TRACKING($dbh, $table, $cfg->{'EDTK_MAX_USER_KEY'}); };
-		if ($@) {
-			warn "INFO : Could not create table : $@\n";
-		}
+			eval { create_table_TRACKING($dbh, $table, $cfg->{'EDTK_MAX_USER_KEY'}); };
+			if ($@) {
+				&logger (4, "Table already exist : $@");
+			}
 	}
 
 	$self->track('Job', 1, join (' ', @ARGV)); # conserver le join pour placer tous les parametres libres dans la zone de message
 	if (defined $cfg->{'EDTK_TRACK_SIG'} && $cfg->{'EDTK_TRACK_SIG'}!~/no/i) {
 		$_TRACK_SIG = $cfg->{'EDTK_TRACK_SIG'};
-		warn "INFO : tracking catchs SIG messages -> '$_TRACK_SIG' set ('warn' for all, 'halt' for die only)\n";
+		&logger (4, "tracking catchs SIG messages = '$_TRACK_SIG' (see EDTK_TRACK_SIG : 'warn' for all, 'halt' for die only)");
 		$_TRACK_TRK = $self;
 	}
+#	if (defined $cfg->{'EDTK_TRACK_JSON'} && $cfg->{'EDTK_TRACK_JSON'} !~/no/i )  {
+#		# $_TRACK_TRK_JSON = $cfg->{'EDTK_TRACK_JSON'};
+#		push (@_TRACK_JSON, $cfg->{'EDTK_TRACK_JSON'});
+#	}
+#	} else {
+#		#$_TRACK_TRK_JSON = "no";
+#		$_TRACK_JSON[0] //= "no";
+#		#push (@_TRACK_JSON, "no");
+#	}
+#	warn "INFO : _TRACK_JSON = " . @_TRACK_JSON ." \n";
+
 	return $self;
 }
 
 
 sub track {
+	# exemple d'appel
+	# $TRK->track('Spool', 1, "K0val", "K1val", "K2val", "Knval" );
 	my ($self, $job, $count, @data) = @_;
 
 	return if $self->{'mode'} eq 'NONE';
@@ -154,7 +194,7 @@ sub track {
 	my @usercols = @{$self->{'keys'}};
 	if (@data > (@usercols +1)) {
 		# max is @usercols nbcol + 1 for message col
-		warn "INFO : Too much values : got " . @data . ", expected " .  (@usercols +1) . " maximum\n";
+		&logger (4, "Too much values : got " . @data . ", expected " .  (@usercols +1) . " maximum");
 	}
 
 	# Validate the job event.
@@ -162,31 +202,38 @@ sub track {
 
 	# GENERATE SQL REQUEST.
 	my $values = {
+		ED_SNGL_ID	=> $self->{'id'},
+		ED_SEQ		=> $self->{'seq'}++,
 		ED_TSTAMP		=> oe_now_time(),
 		ED_USER		=> $self->{'user'},
-		ED_SEQ		=> $self->{'seq'}++,
-		ED_SNGL_ID	=> $self->{'id'},
 		ED_APP		=> $self->{'app'},
+		ED_CORP		=> $self->{'entity'},
+		ED_ACCOUNT	=> $self->{'account'} // "",
 		ED_MOD_ED		=> $self->{'edmode'},
 		ED_JOB_EVT	=> $job,
+		ED_OBJ_TYP	=> $self->{'obj_typ'},
 		ED_OBJ_COUNT	=> $count,
-		ED_CORP		=> $self->{'entity'},
-		ED_SOURCE		=> $self->{'source'},
-		ED_HOST		=> hostname()
+		ED_CHILD_TYP	=> $self->{'child_typ'} // "",
+		ED_CHILD_ID	=> $self->{'child_id'} // "",
+		ED_PARENT_ID	=> $self->{'parent_id'} // "",
+
+		ED_HOST		=> hostname(),
+		ED_SOURCE		=> $self->{'source'}
 	};
 
+
 	foreach my $i (0 .. $#data) {
 		# ajout d'une colonne message pour grer les messages et les warning 
 		# pour assurer la compatibilit avec l'existant on va inverser
 		# les data pour mettre le message en tte en attendant le job_evt
-		##################  PBM  DONNEES NON ALIMENTEES A REGARDER DE PRES
-		my $val = $data[$i] || "";
-		$values->{'ED_MESSAGE'}			= $val . " " . ($values->{'ED_MESSAGE'} || "");
+		##################  PBM  DONNEES NON ALIMENTEES A REGARDER 
+		my $val = $data[$i] // "";
+		$values->{'ED_MESSAGE'}	= $val . " " . ($values->{'ED_MESSAGE'} // "");
 
 		# s'il n'y a qu'une data, on s'assure de ne pas la mettre inutilement dans une colonne utilisateur
 		if ($#data > 0) {
 			if (defined($data[$i]) && length($data[$i]) > 128) {
-				warn "INFO : \"$data[$i]\" truncated to 128 characters\n";
+				&logger (4, "\"$data[$i]\" truncated to 128 characters");
 				$data[$i] =~ s/^(.{128}).*$/$1/;
 			}
 			$values->{"ED_K${i}_NAME"}	= $usercols[$i];
@@ -196,6 +243,7 @@ sub track {
 
 	if ($job eq 'W' || $job eq 'H') { # Halt or Warn event
 		# si le job_evt est 'Warning' ou 'Halt' on gre les messages et la source
+		chomp $values->{'ED_MESSAGE'};
 		$values->{'ED_MESSAGE'}	=~ s/\s+/ /g;
 		$values->{'ED_MESSAGE'}	=~ s/^(.{256}).*$/$1/;
 		$values->{'ED_SOURCE'}	= $self->{'source'} if ($job eq 'H');
@@ -212,6 +260,9 @@ sub track {
 	my $sql	= "INSERT INTO $table (" . join(', ', @cols) . ") VALUES (" .
 	    join(', ', ('?') x @cols) . ")";
 
+	#warn "sql = $sql \n";
+	#sql = INSERT INTO EDTK_TRACKING (ED_JOB_EVT, ED_APP, ED_CORP, ED_OBJ_TYP, ED_USER, ED_MESSAGE, ED_TSTAMP, ED_SNGL_ID, ED_OBJ_COUNT, ED_MOD_ED, ED_SOURCE, ED_SEQ, ED_HOST) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) 
+
 	my $dbh	= $self->{'dbh'};
 	my $sth	= $dbh->prepare($sql);
 	$sth->execute(values(%$values)) or die $sth->errstr;
@@ -219,6 +270,7 @@ sub track {
 	if (!$dbh->{'AutoCommit'}) {
 		$dbh->commit or die $dbh->errstr;
 	}
+	#if ($_TRACK_JSON[0] ne "no") { push (@_TRACK_JSON, $values) };
 }
 
 
@@ -226,7 +278,7 @@ sub set_entity {
 	my ($self, $entity) = @_;
 
 	if (!defined($entity) || length($entity) == 0) {
-		warn "INFO : Tracking::set_entity() called with an undefined entity!\n";
+		&logger (4, "Tracking::set_entity() called with an undefined entity!");
 		return;
 	}
 	# warn "INFO : translate >$entity< \n";
@@ -396,11 +448,11 @@ sub stats_month {
 
 sub _validate_event {
 	# Job Event : looking for one of the following : 
-	#	 Job (default), Spool, Document, Line, Warning, Error, Halt (critic), Reject
+	#	 Job (default), Spool, Document, Line, Warning, Error, Halt (critic), Reject, Track
 	my $job = shift;
 
-	warn "INFO : Halt event in Tracking = $job\n" if ($job =~/^H/);
-	if (!defined $job || $job !~ /^([JSDLWEHR])/) {
+	&logger (3, "Halt event in Tracking = $job") if ($job =~/^H/);
+	if (!defined $job || $job !~ /^([JSDLWEHRT])/) {
 		die "ERROR: Invalid job event : " . (defined $job ? $job : '(undef)') . "\n"
 			. "\t valid events are : Job / Spool / Document / Line / Warning / Reject / Error / Halt (critic)\n"
 			;
@@ -430,4 +482,13 @@ sub _validate_event {
 	}
 #}
 
+
+#END {
+#	#if ($_TRACK_TRK_JSON !~/no/i) {
+#	if ($_TRACK_JSON[0] ne "no") {
+#		my $json = JSON->new;
+#		#print $json->encode(\@_TRACK_JSON);
+#	}
+#}
+
 1;
